suppressMessages(library(getopt))

args = commandArgs(trailingOnly = T)

if (!exists('InFile') || !exists('OutPre')) {
	if (length(args) < 2) {
		message("USAGE: Rscript correlation2.R in_file feature(T/F)\n")
		quit(save='no', status=-1)
	}

	InFile = args[1]
	OutPre = args[2]
	Feature = args[3]
	SamFile = args[4]
	if (length(args) > 4) opt = getopt(matrix(ncol=4, byrow=T, c(
		'pearson', 'p', 2, 'logical',
		'spearman', 's', 2, 'logical'
		)), args[5:length(args)])
	else opt = list()
	if (is.null(opt$pearson)) {
		if (is.null(opt$spearman)) {
			opt$pearson = T
			opt$spearman = T
		} else {
			opt$pearson = F
		}
	} else {
		if (is.null(opt$spearman)) {
			opt$spearman = F
		}
	}
}


suppressMessages(library(geneplotter))
suppressMessages(library(gplots))


openGraphics <- function(path, width, height)
{
	flag = F
	if (exists('PdfFlag')) { if (PdfFlag) flag = T }
	if (!flag) { flag = any(grep("\\.pdf$", path)) }

	if (flag)
	{
		if (!any(grep("\\.pdf$", path))) path <- paste(path, ".pdf", sep="")
		pdf(path, w=width/100, h=height/100)
	}
	else
	{
		if (!any(grep("\\.png$", path))) path <- paste(path, ".png", sep="")
		png(path, w=width, h=height)
	}
}

closeGraphics <- function()
{
	graphics.off()
}

loadFile <- function(file, header=T, ...)
{
	print(paste("Loading", file, ".."))
	dat <- read.table(file, sep="\t", header=header, comment.char='', check.names=F, ...)
	print(head(dat))
	print(tail(dat))
	cat("dim=", dim(dat), "\n")
	return(dat)
}

saveFile <- function(dat, file, rname=F, cname=T)
{
	write.table(dat, quote=F, sep="\t", row.names=rname, col.names=cname, file=file)
}

drawHist <- function(var, file=NA, ...)
{
	if (!is.na(file)) openGraphics(file, 600, 400)
#	r <- range(as.matrix(var))
#	h <- hist(var, breaks=c((r[1]-0.5):(r[2]+0.5)), ...)
	h <- hist(var, ...)
	print(h$counts)
	if (!is.na(file)) closeGraphics()
}

drawHeatmap <- function(x, dist.method, file=NA, ...)
{
	if (!is.na(file)) openGraphics(file, 800, 600)

	hclust2 <- function(x, method="average", ...) hclust(x, method=method, ...)
	dist2 <- function(x, ...) as.dist(1-cor(t(x), method=dist.method))
	if (exists('grpColors')) heatmap.2(as.matrix(x), distfun=dist2, hclustfun=hclust2, scale="none", symm=TRUE, key=TRUE, symkey=FALSE, density.info="density", trace="none", Colv=T, cexRow=1.2, cexCol=1.2, cellnote=x, notecol='black', notecex=1.2, lwid=c(2, 8), lhei=c(2,8), ColSideColors=grpColors, ...)
	else                    heatmap.2(as.matrix(x), distfun=dist2, hclustfun=hclust2, scale="none", symm=TRUE, key=TRUE, symkey=FALSE, density.info="density", trace="none", Colv=T, cexRow=1.2, cexCol=1.2, cellnote=x, notecol='black', notecex=1.2, lwid=c(2, 8), lhei=c(2,8), ...)

	if (!is.na(file)) closeGraphics()
}

draw <- function(fun, x, file=NA, width=600, height=500, ...)
{
	if (!is.na(file)) openGraphics(file, width, height)

	fun(x, ...)

	if (!is.na(file)) closeGraphics()
}

myImagePlot <- function(x, wid=5, ...){
     min <- min(x)
     max <- max(x)
     yLabels <- rownames(x)
     xLabels <- colnames(x)
     title <-c()
	  revFlag <- 0
	  ColorRamp <- NULL
	  xlab <- NULL
	  ylab <- NULL
	  las <- NULL
  # check for additional function arguments
  if( length(list(...)) ){
    Lst <- list(...)
    if( !is.null(Lst$min) ){
       min <- Lst$min
    }
    if( !is.null(Lst$max) ){
       max <- Lst$max
    }
    if( !is.null(Lst$zlim) ){
       min <- Lst$zlim[1]
       max <- Lst$zlim[2]
    }
    if( !is.null(Lst$yLabels) ){
       yLabels <- c(Lst$yLabels)
    }
    if( !is.null(Lst$xLabels) ){
       xLabels <- c(Lst$xLabels)
    }
    if( !is.null(Lst$title) ){
       title <- Lst$title
    }
    if( !is.null(Lst$xlab) ){
       xlab <- Lst$xlab
    } else {
		 xlab <- ''
	 }
    if( !is.null(Lst$ylab) ){
       ylab <- Lst$ylab
    } else {
		 ylab <- ''
	 }
    if( !is.null(Lst$rev) ){
       revFlag <- Lst$rev
    }
    if( !is.null(col) ){
       ColorRamp <- Lst$col
    }
    if( !is.null(Lst$colmap) ){
       ColorRamp <- Lst$colmap(max-min+1)
    }
    if( !is.null(Lst$las) ){
       las <- Lst$las
    }
  }
# check for null values
if( is.null(xLabels) ){
   xLabels <- c(1:ncol(x))
}
if( is.null(yLabels) ){
   yLabels <- c(1:nrow(x))
}

layout(matrix(data=c(1,2), nrow=1, ncol=2), widths=c(wid,1), heights=c(1,1))

 if ( is.null(ColorRamp) )
 {
    # Red and green range from 0 to 1 while Blue ranges from 1 to 0
    ColorRamp <- rgb( seq(0,1,length=256),  # Red
                      seq(0,1,length=256),  # Green
                      seq(1,0,length=256))  # Blue
 }
 ColorLevels <- seq(min, max, length=length(ColorRamp))

 # Reverse Y axis
 if ( revFlag ) {
    reverse <- nrow(x) : 1
    yLabels <- yLabels[reverse]
    x <- x[reverse,]
 }

 # Data Map
 xlen = max(nchar(xLabels))
 ylen = max(nchar(yLabels))
 if ( is.null(xlab) ) {
    mar = c(4,5+xlen/4,2.5,1+ylen/4)
 }
 else {
    mar = c(6,5+xlen/4,2.5,1+ylen/4)
 }
#print(mar)
 par(mar=mar)
 image(1:length(xLabels), 1:length(yLabels), t(x), col=ColorRamp, xlab=xlab, ylab=ylab, axes=FALSE, zlim=c(min,max))
 if( !is.null(title) ){
    title(main=title)
 }
 axis(BELOW<-1, at=1:length(xLabels), labels=xLabels, cex.axis=1.0, tick=F, las=las)
 axis(LEFT <-2, at=1:length(yLabels), labels=yLabels, las= HORIZONTAL<-1, tick=F, cex.axis=1.0)

 # Color Scale
 if ( is.null(xlab) ) {
    par(mar = c(4,1,2.5,3))
 }
 else {
    par(mar = c(6,1,2.5,3))
 }
 image(1, ColorLevels, matrix(data=ColorLevels, ncol=length(ColorLevels),nrow=1), col=ColorRamp, xlab="",ylab="", xaxt="n", yaxt="n")
 axis(4, cex.axis=0.9)

 layout(1)
}

plotPair <- function(logs, title, method, ...)
{
	panel.hist <- function(x, ...)
	{
		usr <- par("usr"); on.exit(par(usr))
		par(usr = c(usr[1:2], 0, 1.5) )
		h <- hist(x, plot=F)
		breaks <- h$breaks; nB <- length(breaks)
		y <- h$counts; y <- y/max(y)
		rect(breaks[-nB], 0, breaks[-1], y, col="gold", ...)
	}

	panel.cor <- function(x, y, digits=2, prefix="", cex.cor, ...)
	{
		usr <- par("usr"); on.exit(par(usr))
		par(usr = c(0, 1, 0, 1))
		c <- cor(x, y, method=method)
		r <- abs(c)
		txt <- format(c(c, 0.123456789), digits=digits)[1]
		txt <- paste(prefix, txt, sep="")
		if(missing(cex.cor)) cex <- 0.8/strwidth(txt)
		text(0.5, 0.5, txt, cex = cex * r, ...)
	}

	panel.scatter <- function(x, y, ...)
	{
		usr <- par("usr"); on.exit(par(usr))
		par(usr = c(usr[1:2], 0, 1.5), new=T)
		smoothScatter(x, y, nrpoints=0, cex=ifelse(PdfFlag, 0.5, 1), ...)
	}

	pairs(logs, main=title, cex.labels=1.5, upper.panel=panel.scatter, lower.panel=panel.cor, labels=colnames(logs), ...)
}

plotPair2 <- function(logs, mat, type, title, method, ...)
{
	panel.scatter <- function(x, y, ...)
	{
		usr <- par("usr"); on.exit(par(usr))
		par(usr = c(usr[1:2], 0, 1.5), new=T)
		smoothScatter(x, y, nrpoints=0, cex=ifelse(PdfFlag, 0.5, 1.2), ...)
	}

#	nf <- layout(matrix(c(1,1,2,2),2,2,byrow=TRUE), widths=c(3.5,3.5), heights=c(2,5), TRUE)
	openGraphics(paste(OutPre, type, method, 'pair1', sep='.'), 700, 400)
	len = max(nchar(colnames(mat)))
	par(mar=c(0,5+len/5,5+len/5,1))
	image(mat[,ncol(mat):1], axes=F)
	text((row(mat)-1)/(nrow(mat)-1), rev(col(mat)-1)/(ncol(mat)-1), mat)
	axis(3, at=(col(mat)[1,]-1)/(ncol(mat)-1), cex.axis=1.2, lab=colnames(mat), las=2)
	axis(2, at=(col(mat)[1,]-1)/(ncol(mat)-1), cex.axis=1.2, lab=rev(rownames(mat)), las=2)
	closeGraphics()
	openGraphics(paste(OutPre, type, method, 'pair2', sep='.'), 700, 500)
	par(mar=c(2,6,1,1))
	pairs(logs, main=title, cex.labels=1.5, upper.panel=panel.scatter, lower.panel=panel.scatter, labels=colnames(logs), xaxt='n', yaxt='n', gap=0)
	closeGraphics()
}

correlationInSample <- function(x, type, method)
{
	ret <- matrix(1, nrow=length(x), ncol=length(x), dimnames=list(colnames(x), colnames(x)))

	for (i in 1:(length(x)-1)) {
		for (j in (i+1):length(x)) {
			ret[i,j] <- round(cor(x[i], x[j], method=method), 2)
			ret[j,i] <- ret[i,j]
		}
	}

	saveFile(ret, paste(OutPre, type, method, 'xls', sep='.'))
	openGraphics(paste(OutPre, type, method, 'mat', sep='.'), 500, 400)
	myImagePlot(ret, min=0, max=1, las=2, col=greenred(100))
	closeGraphics()
	openGraphics(paste(OutPre, type, method, 'pair', sep='.'), 800, 600)
	plotPair(x, NULL, method, gap=0, xaxt='n', yaxt='n')
	closeGraphics()
	plotPair2(x, ret, type, NULL, method)
	drawHeatmap(ret, method, file=paste(OutPre, type, method, 'heatmap', sep='.'))
}


PdfFlag = 0
DatCol = ifelse(Feature, 5, 4)

cat("Start: ", date(), "\n")

dat <- loadFile(InFile)
colnames(dat)[c(1,2,3)] <- c('Ref', 'Start', 'End')
if (DatCol > 4) colnames(dat)[4] <- 'ID'

if (!is.na(SamFile)) {
	sam <- loadFile(SamFile, F, stringsAsFactors=F)
	colnames(sam) = c('file', 'flag', 'sample', 'group')
#	colnames(dat)[DatCol:length(dat)] <- apply(as.matrix(colnames(dat)[DatCol:length(dat)]), 1, function(x) sam$group[x==sam$sample])

#	dat.grp <- apply(as.matrix(as.character(sam$sample)), 1, function(x) sam$group[x==colnames(dat)[DatCol:length(dat)]])
	dat.grp <- apply(as.matrix(colnames(dat[DatCol:length(dat)])), 1, function(x) sam$group[x==sam$sample])
	print(dat.grp)
	stopifnot(length(dat.grp) != 0)
	grpColors <- c('green3', 'gold', 'purple', 'orange', 'magenta', 'cyan')[dat.grp]
}

#flag <- apply(dat, 1, function(x) sum(is.na(x)))
#naFeat <- apply(dat[DatCol:length(dat)], 1, function(x) sum(is.na(x)))
#drawHist(naFeat, freq=F, xlab='Number of NAs', ylab='Number of features', file=paste(OutPre, 'nafeat', sep='.'))

#naSam <- apply(dat[DatCol:length(dat)], 2, function(x) sum(is.na(x)))
#draw(barplot, naSam, ylab='Number of NAs', xlab='Samples', las=2, file=paste(OutPre, 'nasam', sep='.'))

sub <- dat[DatCol:length(dat)]
#union <- sub
#union[is.na(union)] <- 0
common <- na.omit(sub)

#correlationInSample(union, 'union', 'pearson')
#correlationInSample(union, 'union', 'spearman')
if (opt$pearson) correlationInSample(common, 'common', 'pearson')
if (opt$spearman) correlationInSample(common, 'common', 'spearman')

cat("End: ", date(), "\n")
