#!/usr/bin/perl -w
#!/usr/local/bin/perl -w
#!/bin/perl -w

## Author: Jeong-Hyeon Choi
## Date: Oct 14 2008

## join two or more files

## Usage:
## CHECK usage with option  -h

## $Id: perl_template,v 1.2 2007-09-21 22:51:23 jeochoi Exp $


use Getopt::Long qw(:config no_ignore_case);
use File::Basename;

my ($helpFlag, @inFiles, $outFile, $verbose, $quiet, $fieldStr, $keyStr, $valueStr, $missing, $common, $sort, $number, $headerFlag);

GetOptions(
	"h|?|help"		=> \$helpFlag,	
	"input=s"		=> \@inFiles,		## input file
	"output=s"		=> \$outFile,		## output file
	"verbose+"		=> \$verbose,		## verbose output
	"quiet"			=> \$quiet,
	"field=s"		=> \$fieldStr,
	"key=s"			=> \$keyStr,
	"v|value=s"		=> \$valueStr,
	"missing=s"		=> \$missing,
	"common"			=> \$common,
	"sort"			=> \$sort,
	"number"			=> \$number,
	"header"			=> \$headerFlag,
) || die "\n";

checkOptions();

my %hash;
my @sizes;
my @headers;
my $del = '!#:';
my @fields = _parseRangeArg1($fieldStr);
my @keys = split(/,/, $keyStr) if $keyStr;
my @values = _parseRangeArg1($valueStr) if $valueStr;
my @revFields = sort {$b<=>$a} @fields;
my @unknowns = split(/,/, $missing);

foreach my $i (0..$#inFiles)
{
	$unknowns[$i] = $i ? $unknowns[$i-1] : '' if !defined $unknowns[$i];
	load($i);
}

output();

#-------------------------------------------------------------------------------

sub load
{
	my ($i) = @_;
	my $fileName = $inFiles[$i];

	print "Loading ", $fileName || 'STDIN', " ...\n" if !$quiet;
	my $in = openInput($fileName);
	my $flag;

	while (<$in>)
	{
		next if (!$headerFlag && /^#/) || /^\s*$/;
		chop; s/$/a/;
		my @a = split(/[\t\r\n]/);
		$a[-1] =~ s/a$//;

		if (!defined $sizes[$i])
		{
			$sizes[$i] = @a-@fields;
			if ($headerFlag)
			{
				$flag = 1;
				$headers[0] = join("\t", map {s/"//g; $_} @a[@fields]) if $i == 0;
			}
		}
		elsif (@values && $sizes[$i] != @a-@fields)
		{
			die "Inconsistent #fields in $_";
		}

		my $key = join($del, @a[@fields]);
		if (@a == 1)
		{
			$hash{$key}[$i] = 1 if !$flag;
		}
		elsif ($valueStr)
		{
			#push(@{$hash{$key}[$i]}, $a[$value]);
			if ($flag) { $headers[$i+1] = join("\t", map {s/"//g; $_} @a[@values]); }
			else       { $hash{$key}[$i] = join("\t", @a[@values]); }
		}
		else
		{
			#push(@{$hash{$key}[$i]}, @a[0..$field-1]) if $field > 0;
			#push(@{$hash{$key}[$i]}, @a[$field+1..$#a]) if $field < $#a;
			my @b = @a; map { splice(@b, $_, 1) } @revFields;
			if ($flag) { $headers[$i+1] = join("\t", map {s/"//g; $_} @b); }
			else       { $hash{$key}[$i] = join("\t", @b); }
		}

		$flag = 0;
	}

	close($in) if defined $fileName;
}

sub output
{
	my $out = openOutput($outFile);

	if ($headerFlag)
	{
		print $out join("\t", @headers), "\n";
	}
	else
	{
		print $out '#', join("\t", $keyStr ? @keys : (map {"Key".$_} 1..@fields));
		my @files = _getDistinctName(\@inFiles, '[\. ]');
		for (my $i = 0; $i < @files; $i++) { map {print $out "\t$files[$i]"} 1..($valueStr ? @values+0 : ($sizes[$i]||1)); }
		print $out "\n";
	}

	my @keys = keys %hash;
	if ($sort)
	{
		map {$keys[$_] = [split(/$del/, $keys[$_])]} 0..$#keys;
		@keys = sort {($number ? $a->[0] <=> $b->[0] : $a->[0] cmp $b->[0]) || $a->[1] <=> $b->[1]} @keys;
		map {$keys[$_] = join($del, @{$keys[$_]})} 0..$#keys;
	}

	foreach my $key (@keys)
	{
		my $flag;
		my $str = join("\t", split(/$del/, $key));

		foreach my $i (0..$#inFiles)
		{
			my $a = $hash{$key}[$i];
			$str .= "\t";
			if (defined $a) { $str .= $a; }
			elsif (!$common) { $str .= join("\t", map {$unknowns[$i]} 1..($valueStr ? @values+0 : ($sizes[$i]||1))); }
			else { $flag = 1; }
		}

		print $out "$str\n" if !$flag;
	}
}

#-------------------------------------------------------------------------------

sub _getDistinctName
{
	my ($rfiles, $del) = @_;
	$del = '[\._]' if !$del;

	my @arrays;
	map { my @a = split(/$del/, fileparse($_)); push(@arrays, \@a) } @$rfiles;

	my %strs;
	for (my $i = 0; $i < @arrays; $i++)
	{
		map { $strs{$_}{$i}++ } @{$arrays[$i]};
	}

	foreach my $s (keys %strs)
	{
		next if keys(%{$strs{$s}}) < @arrays;
		for (my $i = 0; $i < @arrays; $i++)
		{
			for (my $j = 0; $j < @{$arrays[$i]}; $j++)
			{
				if ($arrays[$i][$j] eq $s) { $arrays[$i][$j] = ''; last; }
			}
		}
	}

	return map {join('', @$_)} @arrays;
}

sub _parseRangeArg1
{
	my @array = _parseRangeArg(@_);
	_decrease(\@array, 1);
	return @array;
}

sub _decrease
{
	my ($array, $value) = @_;
	map {$_-=$value} @$array;
}

sub _parseRangeArg
{
	my %ret;

	foreach my $arg (@_)
	{
		my @a;

		foreach my $r (split(/,/, $arg))
		{
			push(@a, ($r =~ /(\d+)-(\d+)/) ? $1..$2 : $r);
		}

		map {$ret{$_}++} @a;
	}

	return sort {$a<=>$b} keys %ret;
}

sub openInput
{
	my ($fileName) = @_;

	return *STDIN unless defined $fileName;

	my ($fd);
	open($fd, $fileName =~ /.gz(ip)?$/ ? "zcat $fileName |" : $fileName =~ /.bz(ip)?2$/ ? "bzcat $fileName |" : $fileName) || die("Open error: $fileName");
	return $fd;
}

sub openOutput
{
	my ($fileName) = @_;

	return *STDOUT unless defined $fileName;

	my ($fd);
	open($fd, $fileName =~ /.gz$/ ? "| gzip -c > $fileName" : $fileName =~ /.bz(ip)?2$/ ? "| bzip2 -z -c > $fileName" : ">$fileName") || die("Open error: $fileName");
	return $fd;
}

sub checkOptions
{
	push(@inFiles, @ARGV) if !@inFiles  && @ARGV > 0;

	if ($helpFlag || !$fieldStr || !@inFiles)
	{
		die("Arguments: -f key [-header] [-m missing_value] [-v value] [-i] in_file .. [-o out_file] [-v] [-q]\n"
		  . "\t\n"
		  );
	}

	$missing = '' if !defined $missing;
}
